from ctypes import POINTER, c_void_p
import distutils.util
import json
from typing import Any, Callable

from cerver.types import String

from cerver.http import http_query_pairs_get_value
from cerver.http import http_request_get_body

def percepthor_query_value_from_params (
	query: dict, params: c_void_p, name: str, errors: dict
):
	found = http_query_pairs_get_value (params, name.encode ("utf-8"))
	if (found):
		try:
			query[name] = found.contents.str.decode ("utf-8")

		except:
			errors[name] = f"Field {name} is invalid."

	else:
		errors[name] = f"Field {name} is required."

def percepthor_query_value_from_params_with_cast (
	query: dict, params: c_void_p, name: str, cast: Callable [[str], Any], errors: dict
):
	found = http_query_pairs_get_value (params, name.encode ("utf-8"))
	if (found):
		try:
			query[name] = cast (found.contents.str.decode ("utf-8"))

		except:
			errors[name] = f"Field {name} is invalid."

	else:
		errors[name] = f"Field {name} is required."

def percepthor_query_optional_value_from_params (
	query: dict, params: c_void_p, name: str
):
	found = http_query_pairs_get_value (params, name.encode ("utf-8"))
	if (found):
		query[name] = found.contents.str.decode ("utf-8")

def percepthor_query_optional_value_from_params_with_cast (
	query: dict, params: c_void_p, name: str, cast: Callable [[str], Any], errors: dict
):
	found = http_query_pairs_get_value (params, name.encode ("utf-8"))
	if (found):
		try:
			query[name] = cast (found.contents.str.decode ("utf-8"))

		except:
			errors[name] = f"Field {name} is invalid."

def percepthor_query_int_value_from_params (
	query: dict, params: c_void_p, name: str, errors: dict
):
	found = http_query_pairs_get_value (params, name.encode ("utf-8"))
	if (found):
		try:
			query[name] = int (found.contents.str.decode ("utf-8"))
		except ValueError:
			errors[name] = f"Field {name} is invalid."

	else:
		errors[name] = f"Field {name} is required."

def percepthor_query_int_optional_value_from_params (
	query: dict, params: c_void_p, name: str, errors: dict
):
	found = http_query_pairs_get_value (params, name.encode ("utf-8"))
	if (found):
		try:
			query[name] = int (found.contents.str.decode ("utf-8"))
		except ValueError:
			errors[name] = f"Field {name} is invalid."

def percepthor_query_int_value_from_params_with_default (
	query: dict, params: c_void_p, name: str, default: int
):
	query[name] = default

	found = http_query_pairs_get_value (params, name.encode ("utf-8"))
	if (found):
		try:
			query[name] = int (found.contents.str.decode ("utf-8"))
		except ValueError:
			pass

def percepthor_query_float_value_from_params (
	query: dict, params: c_void_p, name: str, errors: dict
):
	found = http_query_pairs_get_value (params, name.encode ("utf-8"))
	if (found):
		try:
			query[name] = float (found.contents.str.decode ("utf-8"))
		except ValueError:
			errors[name] = f"Field {name} is invalid."

	else:
		errors[name] = f"Field {name} is required."

def percepthor_query_float_optional_value_from_params (
	query: dict, params: c_void_p, name: str, errors: dict
):
	found = http_query_pairs_get_value (params, name.encode ("utf-8"))
	if (found):
		try:
			query[name] = float (found.contents.str.decode ("utf-8"))
		except ValueError:
			errors[name] = f"Field {name} is invalid."

def percepthor_query_float_value_from_params_with_default (
	query: dict, params: c_void_p, name: str, default: float
):
	query[name] = default

	found = http_query_pairs_get_value (params, name.encode ("utf-8"))
	if (found):
		try:
			query[name] = float (found.contents.str.decode ("utf-8"))
		except ValueError:
			pass

def percepthor_query_bool_value_from_params (
	query: dict, params: c_void_p, name: str, errors: dict
):
	found = http_query_pairs_get_value (params, name.encode ("utf-8"))
	if (found):
		try:
			query[name] = bool (
				distutils.util.strtobool (found.contents.str.decode ("utf-8"))
			)
		except ValueError:
			errors[name] = f"Field {name} is invalid."

	else:
		errors[name] = f"Field {name} is required."

def percepthor_query_bool_optional_value_from_params (
	query: dict, params: c_void_p, name: str, errors: dict
):
	found = http_query_pairs_get_value (params, name.encode ("utf-8"))
	if (found):
		try:
			query[name] = bool (
				distutils.util.strtobool (found.contents.str.decode ("utf-8"))
			)
		except ValueError:
			errors[name] = f"Field {name} is invalid."

def percepthor_handle_body_input (
	request: c_void_p, handle_body_input: Callable [[dict, dict], dict], errors: dict
) -> dict:
	values = None

	body_json: POINTER (String) = http_request_get_body (request)

	if (body_json is not None):
		loaded_json: dict = json.loads (body_json.contents.str)

		values = handle_body_input (loaded_json, errors)

	else:
		errors["body"] = "Request body input is required!"

	return values



from ctypes import c_void_p, c_char_p, c_int, c_bool

from .lib import lib

PermissionsType = c_int

PERMISSIONS_TYPE_NONE = 0
PERMISSIONS_TYPE_SERVICE = 1
PERMISSIONS_TYPE_ORGANIZATION = 2
PERMISSIONS_TYPE_PROJECT = 3

permissions_type_to_string = lib.permissions_type_to_string
permissions_type_to_string.argtypes = [c_int]
permissions_type_to_string.restype = c_char_p

permissions_get_resource = lib.permissions_get_resource
permissions_get_resource.argtypes = [c_void_p]
permissions_get_resource.restype = c_char_p

permissions_print = lib.permissions_print
permissions_print.argtypes = [c_void_p]

permissions_has_action = lib.permissions_has_action
permissions_has_action.argtypes = [c_void_p, c_char_p]
permissions_has_action.restype = c_bool



from ctypes import c_void_p, c_char_p, c_int, c_uint, c_int64, c_bool

from .lib import lib

PERCEPTHOR_AUTH_TYPE_NONE = 0
PERCEPTHOR_AUTH_TYPE_TOKEN = 1
PERCEPTHOR_AUTH_TYPE_ACTION = 2
PERCEPTHOR_AUTH_TYPE_ROLE = 3
PERCEPTHOR_AUTH_TYPE_SERVICE = 4
PERCEPTHOR_AUTH_TYPE_PERMISSIONS = 5
PERCEPTHOR_AUTH_TYPE_MULTIPLE = 6
PERCEPTHOR_AUTH_TYPE_COMPLETE = 7

PERCEPTHOR_AUTH_SCOPE_NONE = 0
PERCEPTHOR_AUTH_SCOPE_SINGLE = 1
PERCEPTHOR_AUTH_SCOPE_MANAGEMENT = 2

percepthor_auth_type_to_string = lib.percepthor_auth_type_to_string
percepthor_auth_type_to_string.argtypes = [c_int]
percepthor_auth_type_to_string.restype = c_char_p

percepthor_auth_scope_to_string = lib.percepthor_auth_scope_to_string
percepthor_auth_scope_to_string.argtypes = [c_int]
percepthor_auth_scope_to_string.restype = c_char_p

percepthor_auth_delete = lib.percepthor_auth_delete
percepthor_auth_delete.argtypes = [c_void_p]

percepthor_auth_get_type = lib.percepthor_auth_get_type
percepthor_auth_get_type.argtypes = [c_void_p]
percepthor_auth_get_type.restype = c_int

percepthor_auth_get_scope = lib.percepthor_auth_get_scope
percepthor_auth_get_scope.argtypes = [c_void_p]
percepthor_auth_get_scope.restype = c_int

percepthor_auth_get_resource = lib.percepthor_auth_get_resource
percepthor_auth_get_resource.argtypes = [c_void_p]
percepthor_auth_get_resource.restype = c_char_p

percepthor_auth_get_action = lib.percepthor_auth_get_action
percepthor_auth_get_action.argtypes = [c_void_p]
percepthor_auth_get_action.restype = c_char_p

percepthor_auth_get_admin = lib.percepthor_auth_get_admin
percepthor_auth_get_admin.argtypes = [c_void_p]
percepthor_auth_get_admin.restype = c_bool

percepthor_auth_get_permissions = lib.percepthor_auth_get_permissions
percepthor_auth_get_permissions.argtypes = [c_void_p]
percepthor_auth_get_permissions.restype = c_void_p

percepthor_auth_permissions_iter_start = lib.percepthor_auth_permissions_iter_start
percepthor_auth_permissions_iter_start.argtypes = [c_void_p]
percepthor_auth_permissions_iter_start.restype = c_bool

percepthor_auth_permissions_iter_get_next = lib.percepthor_auth_permissions_iter_get_next
percepthor_auth_permissions_iter_get_next.argtypes = [c_void_p]
percepthor_auth_permissions_iter_get_next.restype = c_void_p

percepthor_auth_get_token_id = lib.percepthor_auth_get_token_id
percepthor_auth_get_token_id.argtypes = [c_void_p]
percepthor_auth_get_token_id.restype = c_char_p

percepthor_auth_get_token_type = lib.percepthor_auth_get_token_type
percepthor_auth_get_token_type.argtypes = [c_void_p]
percepthor_auth_get_token_type.restype = c_int

percepthor_auth_get_token_organization = lib.percepthor_auth_get_token_organization
percepthor_auth_get_token_organization.argtypes = [c_void_p]
percepthor_auth_get_token_organization.restype = c_char_p

percepthor_auth_get_token_permissions = lib.percepthor_auth_get_token_permissions
percepthor_auth_get_token_permissions.argtypes = [c_void_p]
percepthor_auth_get_token_permissions.restype = c_char_p

percepthor_auth_get_token_role = lib.percepthor_auth_get_token_role
percepthor_auth_get_token_role.argtypes = [c_void_p]
percepthor_auth_get_token_role.restype = c_char_p

percepthor_auth_get_token_user = lib.percepthor_auth_get_token_user
percepthor_auth_get_token_user.argtypes = [c_void_p]
percepthor_auth_get_token_user.restype = c_char_p

percepthor_auth_get_token_username = lib.percepthor_auth_get_token_username
percepthor_auth_get_token_username.argtypes = [c_void_p]
percepthor_auth_get_token_username.restype = c_char_p

percepthor_auth_get_mask = lib.percepthor_auth_get_mask
percepthor_auth_get_mask.argtypes = [c_void_p]
percepthor_auth_get_mask.restype = c_int64

percepthor_auth_create = lib.percepthor_auth_create
percepthor_auth_create.argtypes = [c_int]
percepthor_auth_create.restype = c_void_p

percepthor_auth_print_token = lib.percepthor_auth_print_token
percepthor_auth_print_token.argtypes = [c_void_p]

percepthor_single_authentication = lib.percepthor_single_authentication
percepthor_single_authentication.argtypes = [c_void_p, c_void_p, c_char_p]
percepthor_single_authentication.restype = c_uint

percepthor_custom_authentication_handler = lib.percepthor_custom_authentication_handler
percepthor_custom_authentication_handler.argtypes = [c_void_p, c_void_p]
percepthor_custom_authentication_handler.restype = c_uint
